__all__ = ['IUnit', 'Unit', 'UnitRef', 'UnitType']

from abc import ABCMeta, abstractmethod
from typing import Generic, Final, TypeVar
from psuctl.firmware.psuctl import PsuctlFirmware
from psuctl.firmware.vitek import VitekFirmware

UnitType = str
UnitRef = tuple[UnitType, str]


class IUnit(metaclass=ABCMeta):
    @abstractmethod
    def run_script(self, conf: dict) -> tuple[int, int] | None:
        pass


TFirmware = TypeVar('TFirmware', bound=PsuctlFirmware | VitekFirmware)


class Unit(IUnit, Generic[TFirmware]):
    def __init__(self, firmware: TFirmware, ref: UnitRef) -> None:
        self._firmware: Final[TFirmware] = firmware
        self._ref: Final[UnitRef] = ref

    @abstractmethod
    def run_script(self, conf: dict) -> tuple[int, int] | None:
        pass
