__all__ = ['sh_unit_factory', 'sh_ics_unit_factory']

from edge import eventlog
from edge.support import sh

from edge.types.exceptions import InternalError
from psuctl.firmware.psuctl import PsuctlFirmware
from psuctl.firmware.vitek import VitekFirmware
from psuctl.types.unit import IUnit


class _ShUnit(IUnit):
    def run_script(self, conf: dict) -> tuple[int, int] | None:
        """
        :param conf:
            psuctl.conf#/scripts/<name>/conf
        """
        shell_cmd = conf.get('shell')
        if shell_cmd and type(shell_cmd) not in [str]:
            eventlog.log_e_param('#/shell')
            raise InternalError

        try:
            rc = sh.run_silent(shell_cmd)
            if rc == 0:
                eventlog.log_d(f'done: {shell_cmd}')
            else:
                eventlog.log_d(f'fail: {shell_cmd} rc {rc}')

        except Exception:
            eventlog.log_x()
            raise InternalError

        return None


class _ShIcsUnit(IUnit):
    def run_script(self, conf: dict) -> tuple[int, int] | None:
        """
        :param conf:
            psuctl.conf#/scripts/<name>/conf
        """
        action = conf.get('action')

        if action not in ['rotate', 'get-limits']:
            eventlog.log_e_param('#/action')
            raise InternalError

        if action == 'get-limits':
            ch_max = conf.get('max-steps')
            ch_min = conf.get('min-steps', 0)
            if ch_max is None:
                eventlog.log_e('fail:#/max-steps')
                raise InternalError
            return ch_min, ch_max

        raw_steps = conf.get('steps')
        if type(raw_steps) in [str]:
            raw_steps = int(raw_steps)
        elif type(raw_steps) in [int]:
            pass
        else:
            eventlog.log_e_param('#/steps')
            raise InternalError

        # rot_steps = abs(raw_steps)

        shell_cmd = conf.get('shell')
        if shell_cmd and type(shell_cmd) not in [str]:
            eventlog.log_e_param('#/shell')
            raise InternalError

        shell_cmd += f' {raw_steps}'
        try:
            rc = sh.run_silent(shell_cmd)
            if rc == 0:
                eventlog.log_d(f'done: {shell_cmd}')
            else:
                eventlog.log_d(f'fail: {shell_cmd} rc {rc}')
                raise InternalError

        except Exception:
            eventlog.log_x()
            raise InternalError


# noinspection PyUnusedLocal
def sh_unit_factory(
        firmware: PsuctlFirmware | VitekFirmware | None,
        unit_ref: tuple[str, str] | None = None
        ) -> IUnit:

    return _ShUnit()


# noinspection PyUnusedLocal
def sh_ics_unit_factory(
        firmware: PsuctlFirmware | VitekFirmware | None,
        unit_ref: tuple[str, str] | None = None
        ) -> IUnit:

    return _ShIcsUnit()
